@@ -16,43 +16,15 @@ import platform
 from xpra.platform.graph import make_graph_pixmap
 from xpra.deque import maxdeque
 from xpra.stats.base import values_to_scaled_values, values_to_diff_scaled_values, to_std_unit, std_unit_dec
+from xpra.scripts.config import HAS_SOUND, ENCODINGS
 from wimpiggy.log import Logger
-from xpra.platform.client_extras_base import set_tooltip_text
+from xpra.gtk_util import add_close_accel, label, title_box, set_tooltip_text, TableBuilder
 log = Logger()
 
 N_SAMPLES = 20      #how many sample points to show on the graphs
 SHOW_PIXEL_STATS = True
 
 
-def add_close_accel(window, callback):
-    if is_gtk3():
-        return      #TODO: implement accel for gtk3
-    # key accelerators
-    accel_group = gtk.AccelGroup()
-    accel_group.connect_group(ord('w'), gdk.CONTROL_MASK, gtk.ACCEL_LOCKED, callback)
-    window.add_accel_group(accel_group)
-    accel_group = gtk.AccelGroup()
-    escape_key, modifier = gtk.accelerator_parse('Escape')
-    accel_group.connect_group(escape_key, modifier, gtk.ACCEL_LOCKED |  gtk.ACCEL_VISIBLE, callback)
-    window.add_accel_group(accel_group)
-
-def label(text="", tooltip=None):
-    l = gtk.Label(text)
-    if tooltip:
-        set_tooltip_text(l, tooltip)
-    return l
-
-def title_box(label_str):
-    eb = gtk.EventBox()
-    l = label(label_str)
-    l.modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color(red=48, green=0, blue=0))
-    al = gtk.Alignment(xalign=0.0, yalign=0.5, xscale=0.0, yscale=0.0)
-    al.set_padding(0, 0, 10, 10)
-    al.add(l)
-    eb.add(al)
-    eb.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(red=219, green=226, blue=242))
-    return eb
-
 def pixelstr(v):
     if v<0:
         return  "n/a"
@@ -71,7 +43,7 @@ def average(seconds, pixel_counter):
     avgs = 0
     mint = now-seconds      #ignore records older than N seconds
     startt = now            #when we actually start counting from
-    for t, count in pixel_counter:
+    for _, t, count in pixel_counter:
         if t>=mint:
             total += count
             total_n += 1
@@ -89,44 +61,6 @@ def average(seconds, pixel_counter):
     return int(total/elapsed), total_n/elapsed, mins, avgs, maxs
 
 
-class TableBuilder(object):
-
-    def __init__(self, rows=1, columns=2, homogeneous=False):
-        self.table = gtk.Table(rows, columns, homogeneous)
-        self.table.set_col_spacings(0)
-        self.table.set_row_spacings(0)
-        self.row = 0
-        self.widget_xalign = 0.0
-
-    def get_table(self):
-        return self.table
-
-    def add_row(self, label, *widgets):
-        if label:
-            l_al = gtk.Alignment(xalign=1.0, yalign=0.5, xscale=0.0, yscale=0.0)
-            l_al.add(label)
-            self.attach(l_al, 0)
-        if widgets:
-            i = 1
-            for w in widgets:
-                if w:
-                    w_al = gtk.Alignment(xalign=self.widget_xalign, yalign=0.5, xscale=0.0, yscale=0.0)
-                    w_al.add(w)
-                    self.attach(w_al, i)
-                i += 1
-        self.inc()
-
-    def attach(self, widget, i, count=1, xoptions=gtk.FILL, xpadding=10):
-        self.table.attach(widget, i, i+count, self.row, self.row+1, xoptions=xoptions, xpadding=xpadding)
-
-    def inc(self):
-        self.row += 1
-
-    def new_row(self, row_label_str, value1, value2=None, label_text=None):
-        row_label = label(row_label_str, label_text)
-        self.add_row(row_label, value1, value2)
-
-
 class SessionInfo(gtk.Window):
 
     def __init__(self, client, session_name, window_icon_pixbuf, conn, get_pixbuf):
@@ -195,7 +129,6 @@ class SessionInfo(gtk.Window):
             tb.new_row("GTK", label(client_version_info("gtk_version")), label(server_version_info("gtk_version")))
         tb.new_row("Python", label(platform.python_version()), label(server_version_info("python_version")))
         cl_gst_v, cl_pygst_v = "", ""
-        from xpra.scripts.main import HAS_SOUND
         if HAS_SOUND:
             try:
                 from xpra.sound.gstreamer_util import gst_version as cl_gst_v, pygst_version as cl_pygst_v
@@ -215,6 +148,12 @@ class SessionInfo(gtk.Window):
         randr_box.add(self.server_randr_icon)
         randr_box.add(self.server_randr_label)
         tb.new_row("RandR Support", randr_box)
+        opengl_box = gtk.HBox(False, 20)
+        self.server_opengl_label = label()
+        self.server_opengl_icon = gtk.Image()
+        opengl_box.add(self.server_opengl_icon)
+        opengl_box.add(self.server_opengl_label)
+        tb.new_row("Client OpenGL", opengl_box)
         self.server_encodings_label = label()
         tb.new_row("Server Encodings", self.server_encodings_label)
         self.client_encodings_label = label()
@@ -260,7 +199,7 @@ class SessionInfo(gtk.Window):
         if self.client.server_platform:
             tb.new_row("Server Platform", label(self.client.server_platform))
         self.server_load_label = label()
-        tb.new_row("Server Load", self.server_load_label, label_text="Average over 1, 5 and 15 minutes")
+        tb.new_row("Server Load", self.server_load_label, label_tooltip="Average over 1, 5 and 15 minutes")
         self.session_started_label = label()
         tb.new_row("Session Started", self.session_started_label)
         self.session_connected_label = label()
@@ -306,6 +245,8 @@ class SessionInfo(gtk.Window):
                 self.damage_labels = maths_labels()
                 tb.add_row(label("Damage Latency (ms)"), *self.damage_labels)
 
+            self.decoding_labels = maths_labels()
+            tb.add_row(label("Decoding Latency (ms)"), *self.decoding_labels)
             self.regions_per_second_labels = maths_labels()
             tb.add_row(label("Regions/s"), *self.regions_per_second_labels)
             self.regions_sizes_labels = maths_labels()
@@ -444,7 +385,7 @@ class SessionInfo(gtk.Window):
         self.net_out_data.append(self.connection.output_bytecount)
         #count pixels in the last second:
         since = time.time()-1
-        decoded = [0]+[pixels for t,pixels in self.client.pixel_counter if t>since]
+        decoded = [0]+[pixels for _,t,pixels in self.client.pixel_counter if t>since]
         self.pixel_in_data.append(sum(decoded))
         #update latency values
         self.server_latency = [1000.0*x for _,x in list(self.client.server_ping_latency)[-20:]]
@@ -464,7 +405,6 @@ class SessionInfo(gtk.Window):
         return False
 
     def populate_features(self):
-        from xpra.scripts.main import ENCODINGS
         size_info = ""
         if self.client.server_actual_desktop_size:
             w,h = self.client.server_actual_desktop_size
@@ -473,6 +413,13 @@ class SessionInfo(gtk.Window):
                 size_info += " (max %s)" % ("x".join([str(x) for x in self.client.server_max_desktop_size]))
         self.bool_icon(self.server_randr_icon, self.client.server_randr)
         self.server_randr_label.set_text("%s" % size_info)
+        self.bool_icon(self.server_opengl_icon, self.client.opengl_enabled)
+        if self.client.opengl_enabled:
+            glinfo = "%s / %s" % (self.client.opengl_props.get("vendor", ""), self.client.opengl_props.get("renderer", ""))
+        else:
+            glinfo = self.client.opengl_props.get("info", "")
+        self.server_opengl_label.set_text(glinfo)
+        
         scaps = self.client.server_capabilities
         self.server_encodings_label.set_text(", ".join(scaps.get("encodings", [])))
         self.client_encodings_label.set_text(", ".join(ENCODINGS))
@@ -590,21 +537,24 @@ class SessionInfo(gtk.Window):
                     return getv(prefix+".cur"), getv(prefix+".min"), getv(prefix+".avg"), getv(prefix+".90p"), getv(prefix+".max")
                 setall(self.batch_labels, values_from_info("batch_delay"))
                 setall(self.damage_labels, values_from_info("damage_out_latency"))
+
             region_sizes = []
             rps = []
             pps = []
+            decoding_latency = []
             if len(self.client.pixel_counter)>0:
                 min_time = None
                 max_time = None
                 regions_per_second = {}
                 pixels_per_second = {}
-                for event_time, size in self.client.pixel_counter:
+                for start_time, end_time, size in self.client.pixel_counter:
+                    decoding_latency.append(int(1000.0*(end_time-start_time)))
                     region_sizes.append(size)
-                    if min_time is None or min_time>event_time:
-                        min_time = event_time
-                    if max_time is None or max_time<event_time:
-                        max_time = event_time
-                    time_in_seconds = int(event_time)
+                    if min_time is None or min_time>end_time:
+                        min_time = end_time
+                    if max_time is None or max_time<end_time:
+                        max_time = end_time
+                    time_in_seconds = int(end_time)
                     regions = regions_per_second.get(time_in_seconds, 0)
                     regions_per_second[time_in_seconds] = regions+1
                     pixels = pixels_per_second.get(time_in_seconds, 0)
@@ -612,6 +562,7 @@ class SessionInfo(gtk.Window):
                 for t in xrange(int(min_time), int(max_time+1)):
                     rps.append(regions_per_second.get(t, 0))
                     pps.append(pixels_per_second.get(t, 0))
+            setlabels(self.decoding_labels, decoding_latency)
             setlabels(self.regions_per_second_labels, rps)
             setlabels(self.regions_sizes_labels, region_sizes, rounding=std_unit_dec)
             setlabels(self.pixels_per_second_labels, pps, rounding=std_unit_dec)
@@ -722,5 +673,7 @@ class SessionInfo(gtk.Window):
             log.warn("unknown chooser response: %d" % response)
 
     def destroy(self, *args):
+        log("SessionInfo.destroy(%s) is_closed=%s", args, self.is_closed)
         self.is_closed = True
         gtk.Window.destroy(self)
+        log("SessionInfo.destroy(%s) done", args)
