@@ -12,8 +12,6 @@
 #   any other interesting metadata? _NET_WM_TYPE, WM_TRANSIENT_FOR, etc.?
 
 import gtk.gdk
-gtk.gdk.threads_init()
-
 import gobject
 
 try:
@@ -40,7 +38,7 @@ from wimpiggy.log import Logger
 log = Logger()
 
 import xpra
-from xpra.server_base import XpraServerBase
+from xpra.x11_server_base import X11ServerBase
 from xpra.pixbuf_to_rgb import get_rgb_rawdata
 from xpra.protocol import zlib_compress, Compressed
 
@@ -136,10 +134,10 @@ class DesktopManager(gtk.Widget):
         transient_for = window.get_property("transient-for")
         if transient_for is None:
             return None
-        log("found transient_for=%s, xid=%s", transient_for, transient_for.xid)
+        log("found transient_for=%s, xid=%s", transient_for, hex(transient_for.xid))
         #try to find the model for this window:
         for model in self._models.keys():
-            log("testing model %s: %s", model, model.client_window.xid)
+            log("testing model %s: %s", model, hex(model.client_window.xid))
             if model.client_window.xid==transient_for.xid:
                 wid = window_to_id.get(model)
                 log("found match, window id=%s", wid)
@@ -147,31 +145,14 @@ class DesktopManager(gtk.Widget):
         root = gtk.gdk.get_default_root_window()
         if root.xid==transient_for.xid:
             return -1       #-1 is the backwards compatible marker for root...
-        log.info("not found transient_for=%s, xid=%s", transient_for, transient_for.xid)
+        log.info("not found transient_for=%s, xid=%s", transient_for, hex(transient_for.xid))
         return  None
 
 
 gobject.type_register(DesktopManager)
 
-def window_name(window):
-    from wimpiggy.prop import prop_get
-    return prop_get(window, "_NET_WM_NAME", "utf8", True) or "unknown"
 
-def window_info(window):
-    from wimpiggy.prop import prop_get
-    net_wm_name = prop_get(window, "_NET_WM_NAME", "utf8", True)
-    return "%s %s (%s / %s)" % (net_wm_name, window, window.get_geometry(), window.is_visible())
-
-def dump_windows():
-    root = gtk.gdk.get_default_root_window()
-    log("root window: %s" % root)
-    children = get_children(root)
-    log("%s windows" % len(children))
-    for window in get_children(root):
-        log("found window: %s", window_info(window))
-
-
-class XpraServer(gobject.GObject, XpraServerBase):
+class XpraServer(gobject.GObject, X11ServerBase):
     __gsignals__ = {
         "wimpiggy-child-map-event": one_arg_signal,
         "wimpiggy-cursor-event": one_arg_signal,
@@ -179,11 +160,11 @@ class XpraServer(gobject.GObject, XpraServerBase):
 
     def __init__(self, clobber, sockets, opts):
         gobject.GObject.__init__(self)
-        XpraServerBase.__init__(self, clobber, sockets, opts)
+        X11ServerBase.__init__(self, clobber, sockets, opts)
 
     def x11_init(self, clobber):
+        X11ServerBase.x11_init(self, clobber)
         init_x11_filter()
-        self.init_x11_atoms()
 
         self._has_focus = 0
         # Do this before creating the Wm object, to avoid clobbering its
@@ -223,6 +204,12 @@ class XpraServer(gobject.GObject, XpraServerBase):
     def is_shown(self, window):
         return self._desktop_manager.is_shown(window)
 
+    def cleanup(self, *args):
+        if self._tray:
+            self._tray.cleanup()
+            self._tray = None
+        X11ServerBase.cleanup(self)
+
     def load_existing_windows(self, system_tray):
         # Tray handler:
         if system_tray:
@@ -274,13 +261,15 @@ class XpraServer(gobject.GObject, XpraServerBase):
             elif window.is_OR():
                 #code more or less duplicated from _send_new_or_window_packet:
                 x, y, w, h = window.get_property("geometry")
-                ss.new_window("new-override-redirect", wid, window, x, y, w, h, self._OR_metadata, self.client_properties.get(ss.uuid))
+                wprops = self.client_properties.get("%s|%s" % (wid, ss.uuid))
+                ss.new_window("new-override-redirect", wid, window, x, y, w, h, self._OR_metadata, wprops)
                 ss.damage(wid, window, 0, 0, w, h)
             else:
                 #code more or less duplicated from send_new_window_packet:
                 self._desktop_manager.hide_window(window)
                 x, y, w, h = self._desktop_manager.window_geometry(window)
-                ss.new_window("new-window", wid, window, x, y, w, h, self._all_metadata, self.client_properties.get(ss.uuid))
+                wprops = self.client_properties.get("%s|%s" % (wid, ss.uuid))
+                ss.new_window("new-window", wid, window, x, y, w, h, self._all_metadata, wprops)
         ss.send_cursor(self.cursor_data)
 
 
@@ -302,9 +291,9 @@ class XpraServer(gobject.GObject, XpraServerBase):
             self._add_new_or_window(event.window)
 
     def _add_new_window_common(self, window):
-        wid = XpraServerBase._add_new_window_common(self, window)
-        window.connect("client-contents-changed", self._contents_changed)
-        window.connect("unmanaged", self._lost_window)
+        wid = X11ServerBase._add_new_window_common(self, window)
+        window.managed_connect("client-contents-changed", self._contents_changed)
+        window.managed_connect("unmanaged", self._lost_window)
         return wid
 
     _window_export_properties = ("title", "size-hints")
@@ -312,27 +301,35 @@ class XpraServer(gobject.GObject, XpraServerBase):
         self._add_new_window_common(window)
         for prop in self._window_export_properties:
             window.connect("notify::%s" % prop, self._update_metadata)
-        (x, y, w, h, _) = window.get_property("client-window").get_geometry()
+        _, _, w, h, _ = window.get_property("client-window").get_geometry()
+        x, y, _, _, _ = window.corral_window.get_geometry()
         log("Discovered new ordinary window: %s (geometry=%s)", window, (x, y, w, h))
         self._desktop_manager.add_window(window, x, y, w, h)
         self._send_new_window_packet(window)
 
     def _add_new_or_window(self, raw_window):
+        xid = get_xwindow(raw_window)
+        if raw_window.get_window_type()==gtk.gdk.WINDOW_TEMP:
+            #ignoring one of gtk's temporary windows
+            #all the windows we manage should be gtk.gdk.WINDOW_FOREIGN
+            log("ignoring TEMP window %s", hex(xid))
+            return
         WINDOW_MODEL_KEY = "_xpra_window_model_"
         wid = raw_window.get_data(WINDOW_MODEL_KEY)
         window = self._id_to_window.get(wid)
-        xid = get_xwindow(raw_window)
         if window:
             if window.is_managed():
-                log("found existing window model %s for %s, will refresh it", type(window), xid)
+                log("found existing window model %s for %s, will refresh it", type(window), hex(xid))
                 geometry = window.get_property("geometry")
                 _, _, w, h = geometry
                 self._damage(window, 0, 0, w, h, options={"min_delay" : 50})
                 return
-            log("found existing model %s (but no longer managed!) for %s", type(window), xid)
-            return
+            log("found existing model %s (but no longer managed!) for %s", type(window), hex(xid))
+            #TODO: we could try to re-use the existing model and window ID,
+            #but for now it is just easier to create a new one:
+            self._lost_window(window)
         tray_window = get_tray_window(raw_window)
-        log("Discovered new override-redirect window: %s (tray=%s)", xid, tray_window)
+        log("Discovered new override-redirect window: %s (tray=%s)", hex(xid), tray_window)
         try:
             if tray_window is not None:
                 assert self._tray
@@ -347,6 +344,7 @@ class XpraServer(gobject.GObject, XpraServerBase):
                 raw_window.set_data(WINDOW_MODEL_KEY, wid)
                 window.call_setup()
                 window.connect("notify::geometry", self._or_window_geometry_changed)
+                window.connect("geometry", self._or_window_geometry_changed)
                 self._send_new_or_window_packet(window)
         except Unmanageable, e:
             if window:
@@ -354,11 +352,11 @@ class XpraServer(gobject.GObject, XpraServerBase):
                 #so we need to fail it manually:
                 window.setup_failed(e)
             else:
-                log.warn("cannot add %s: %s", xid, e)
+                log.warn("cannot add %s: %s", hex(xid), e)
             #from now on, we return to the gtk main loop,
             #so we *should* get a signal when the window goes away
 
-    def _or_window_geometry_changed(self, window, pspec):
+    def _or_window_geometry_changed(self, window, pspec=None):
         (x, y, w, h) = window.get_property("geometry")
         log("or_window_geometry_changed: %s (window=%s)", window.get_property("geometry"), window)
         wid = self._window_to_id[window]
@@ -486,7 +484,7 @@ class XpraServer(gobject.GObject, XpraServerBase):
         self._desktop_manager.show_window(window)
         self._damage(window, 0, 0, width, height)
         if len(packet)>=7:
-            self._set_client_properties(proto, packet[6])
+            self._set_client_properties(proto, wid, packet[6])
 
 
     def _process_unmap_window(self, proto, packet):
@@ -516,7 +514,7 @@ class XpraServer(gobject.GObject, XpraServerBase):
         if self._desktop_manager.visible(window) and (oww!=w or owh!=h):
             self._damage(window, 0, 0, w, h)
         if len(packet)>=7:
-            self._set_client_properties(proto, packet[6])
+            self._set_client_properties(proto, wid, packet[6])
 
     def _process_move_window(self, proto, packet):
         wid, x, y = packet[1:4]
@@ -548,6 +546,8 @@ class XpraServer(gobject.GObject, XpraServerBase):
 
     def _process_mouse_common(self, proto, wid, pointer, modifiers):
         ss = self._server_sources.get(proto)
+        if ss is None:
+            return      #gone already!
         ss.make_keymask_match(modifiers)
         window = self._id_to_window.get(wid)
         if not window:
@@ -556,7 +556,7 @@ class XpraServer(gobject.GObject, XpraServerBase):
         def raise_and_move():
             self._desktop_manager.raise_window(window)
             self._move_pointer(pointer)
-        trap.swallow_unsynced(raise_and_move)
+        trap.swallow_synced(raise_and_move)
 
 
     def _process_close_window(self, proto, packet):
@@ -569,45 +569,62 @@ class XpraServer(gobject.GObject, XpraServerBase):
 
 
     def make_screenshot_packet(self):
+        return trap.call_synced(self.do_make_screenshot_packet)
+
+    def do_make_screenshot_packet(self):
         log("grabbing screenshot")
         regions = []
+        OR_regions = []
         for wid in reversed(sorted(self._id_to_window.keys())):
-            window = self._id_to_window[wid]
+            window = self._id_to_window.get(wid)
+            log("screenshot: window(%s)=%s", wid, window)
+            if window is None or window.is_tray() or not window.is_managed():
+                continue
             pixmap = window.get_property("client-contents")
+            log("screenshot: pixmap(%s)=%s", window, pixmap)
             if pixmap is None:
                 continue
-            (x, y, _, _) = self._desktop_manager.window_geometry(window)
+            if window.is_OR():
+                x, y = window.get_property("geometry")[:2]
+            else:
+                x, y = self._desktop_manager.window_geometry(window)[:2]
+            log("screenshot: position(%s)=%s,%s", window, x, y)
             w, h = pixmap.get_size()
+            log("screenshot: size(%s)=%sx%s", pixmap, w, h)
             item = (wid, x, y, w, h, pixmap)
-            if self._has_focus==wid:
+            if window.is_OR():
+                OR_regions.append(item)
+            elif self._has_focus==wid:
                 #window with focus first (drawn last)
                 regions.insert(0, item)
             else:
                 regions.append(item)
-        log("screenshot: found regions=%s", regions)
-        if len(regions)==0:
-            packet = ["screenshot", 0, 0, "png", -1, ""]
-        else:
-            minx = min([x for (_,x,_,_,_,_) in regions])
-            miny = min([y for (_,_,y,_,_,_) in regions])
-            maxx = max([(x+w) for (_,x,_,w,_,_) in regions])
-            maxy = max([(y+h) for (_,_,y,_,h,_) in regions])
-            width = maxx-minx
-            height = maxy-miny
-            log("screenshot: %sx%s, min x=%s y=%s", width, height, minx, miny)
-            import Image
-            image = Image.new("RGBA", (width, height))
-            for wid, x, y, w, h, pixmap in reversed(regions):
-                _, _, wid, _, _, w, h, _, raw_data, rowstride, _, _ = get_rgb_rawdata(0, 0, wid, pixmap, 0, 0, w, h, "rgb24", -1, None, logger=log.debug)
-                window_image = Image.fromstring("RGB", (w, h), raw_data, "raw", "RGB", rowstride)
-                tx = x-minx
-                ty = y-miny
-                image.paste(window_image, (tx, ty))
-            buf = StringIO()
-            image.save(buf, "png")
-            data = buf.getvalue()
-            buf.close()
-            packet = ["screenshot", width, height, "png", rowstride, Compressed("png", data)]
+        all_regions = OR_regions+regions
+        if len(all_regions)==0:
+            log("screenshot: no regions found, returning empty 0x0 image!")
+            return ["screenshot", 0, 0, "png", -1, ""]
+        log("screenshot: found regions=%s, OR_regions=%s", regions, OR_regions)
+        minx = min([x for (_,x,_,_,_,_) in all_regions])
+        miny = min([y for (_,_,y,_,_,_) in all_regions])
+        maxx = max([(x+w) for (_,x,_,w,_,_) in all_regions])
+        maxy = max([(y+h) for (_,_,y,_,h,_) in all_regions])
+        width = maxx-minx
+        height = maxy-miny
+        log("screenshot: %sx%s, min x=%s y=%s", width, height, minx, miny)
+        import Image
+        image = Image.new("RGBA", (width, height))
+        for wid, x, y, w, h, pixmap in reversed(all_regions):
+            _, _, wid, _, _, w, h, _, raw_data, rowstride, _, _ = get_rgb_rawdata(0, 0, wid, pixmap, 0, 0, w, h, "rgb24", -1, None, logger=log.debug)
+            window_image = Image.fromstring("RGB", (w, h), raw_data, "raw", "RGB", rowstride)
+            tx = x-minx
+            ty = y-miny
+            image.paste(window_image, (tx, ty))
+        buf = StringIO()
+        image.save(buf, "png")
+        data = buf.getvalue()
+        buf.close()
+        packet = ["screenshot", width, height, "png", rowstride, Compressed("png", data)]
+        log("screenshot: %sx%s %s", packet[1], packet[2], packet[-1])
         return packet
 
 
