@@ -28,10 +28,7 @@ AUTO_REFRESH_SPEED = int(os.environ.get("XPRA_AUTO_REFRESH_SPEED", 0))
 #(cannot be lower than DamageBatchConfig.MAX_EVENTS)
 NRECS = 100
 
-
 import gtk.gdk
-gtk.gdk.threads_init()
-
 import gobject
 try:
     from StringIO import StringIO   #@UnusedImport
@@ -42,9 +39,6 @@ import ctypes
 from threading import Lock
 from math import sqrt
 
-#it would be nice to be able to get rid of this import here:
-from wimpiggy.lowlevel import get_rectangle_from_region   #@UnresolvedImport
-
 from wimpiggy.log import Logger
 log = Logger()
 
@@ -64,7 +58,7 @@ warn = log.warn
 
 from xpra.deque import maxdeque
 from xpra.protocol import zlib_compress, Compressed
-from xpra.scripts.main import ENCODINGS
+from xpra.scripts.config import ENCODINGS
 from xpra.pixbuf_to_rgb import get_rgb_rawdata
 from xpra.stats.base import add_list_stats, add_weighted_list_stats
 from xpra.stats.maths import logp, \
@@ -73,6 +67,30 @@ from xpra.stats.maths import logp, \
 from xpra.batch_delay_calculator import calculate_batch_delay, update_video_encoder
 from xpra.xor import xor_str        #@UnresolvedImport
 
+#old gtk versions lack gtk.gdk.Region().get_rectangles()
+#so for those we just keep them in a list..
+#(which isn't as good since we don't merge rectangles
+#or discard subsets, but better than carrying ugly crufty code
+#just for those outdated pygtk versions..)
+tmp_region = gtk.gdk.Region()
+if hasattr(tmp_region, "get_rectangles") and os.environ.get("XPRA_FAKE_OLD_PYGTK", "0")=="0":
+    def new_region():
+        return gtk.gdk.Region()
+    def add_rectangle(region, rectangle):
+        region.union_with_rect(rectangle)
+    def get_rectangles(region):
+        return region.get_rectangles()
+else:
+    log.warn("using get_rectangles workaround for old pygtk versions")
+    def new_region():
+        return list()
+    def add_rectangle(region, rectangle):
+        if rectangle not in region:
+            region.append(rectangle)
+    def get_rectangles(region):
+        return region
+del tmp_region
+
 
 class DamageBatchConfig(object):
     """
@@ -235,7 +253,7 @@ class WindowPerformanceStatistics(object):
             factors.append((msg, target, weight))
         return factors
 
-    def add_stats(self, info, suffix=""):
+    def add_stats(self, info, prefix, suffix=""):
         #encoding stats:
         if len(self.encoding_stats)>0:
             estats = list(self.encoding_stats)
@@ -252,23 +270,23 @@ class WindowPerformanceStatistics(object):
                         comp_times_ns.append((1000.0*1000*1000*compression_time/pixels, pixels))
                         total_pixels += pixels
                         total_time += compression_time
-                add_weighted_list_stats(info, "compression_ratio_pct"+suffix, comp_ratios_pct)
-                add_weighted_list_stats(info, "compression_pixels_per_ns"+suffix, comp_times_ns)
+                add_weighted_list_stats(info, prefix+"compression_ratio_pct"+suffix, comp_ratios_pct)
+                add_weighted_list_stats(info, prefix+"compression_pixels_per_ns"+suffix, comp_times_ns)
                 if total_time>0:
-                    info["pixels_encoded_per_second"+suffix] = int(total_pixels / total_time)
+                    info[prefix+"pixels_encoded_per_second"+suffix] = int(total_pixels / total_time)
             add_compression_stats(estats, suffix=suffix)
             for encoding in encodings_used:
                 enc_stats = [x for x in estats if x[0]==encoding]
                 add_compression_stats(enc_stats, suffix="%s[%s]" % (suffix, encoding))
 
         latencies = [x*1000 for _, _, _, x in list(self.damage_in_latency)]
-        add_list_stats(info, "damage_in_latency",  latencies)
+        add_list_stats(info, prefix+"damage_in_latency",  latencies, show_percentile=[9])
         latencies = [x*1000 for _, _, _, x in list(self.damage_out_latency)]
-        add_list_stats(info, "damage_out_latency",  latencies)
+        add_list_stats(info, prefix+"damage_out_latency",  latencies, show_percentile=[9])
         #per encoding totals:
         for encoding, totals in self.encoding_totals.items():
-            info["total_frames%s[%s]" % (suffix, encoding)] = totals[0]
-            info["total_pixels%s[%s]" % (suffix, encoding)] = totals[1]
+            info[prefix+"total_frames%s[%s]" % (suffix, encoding)] = totals[0]
+            info[prefix+"total_pixels%s[%s]" % (suffix, encoding)] = totals[1]
 
     def get_target_client_latency(self, min_client_latency, avg_client_latency, abs_min=0.010):
         """ geometric mean of the minimum (+20%) and average latency
@@ -478,7 +496,7 @@ class WindowSource(object):
                     continue
                 info[prefix+k] = v
         info[prefix+"encoding"+suffix] = self.encoding
-        self.statistics.add_stats(info, suffix)
+        self.statistics.add_stats(info, prefix, suffix)
         #batch stats:
         if len(self.batch_config.last_actual_delays)>0:
             batch_delays = [x for _,x in list(self.batch_config.last_delays)]
@@ -498,7 +516,7 @@ class WindowSource(object):
 
     def calculate_batch_delay(self):
         calculate_batch_delay(self.window_dimensions, self.wid, self.batch_config, self.global_statistics, self.statistics)
-    
+
     def update_video_encoder(self):
         if self._video_encoder and not self._video_encoder.is_closed():
             update_video_encoder(self.wid, self.window_dimensions, self.batch_config, self.global_statistics, self.statistics,
@@ -534,7 +552,7 @@ class WindowSource(object):
         if self._damage_delayed:
             #use existing delayed region:
             region = self._damage_delayed[2]
-            region.union_with_rect(gtk.gdk.Rectangle(x, y, w, h))
+            add_rectangle(region, gtk.gdk.Rectangle(x, y, w, h))
             #merge/override options
             if options is not None:
                 override = options.get("override_options", False)
@@ -573,8 +591,8 @@ class WindowSource(object):
             return
 
         #create a new delayed region:
-        region = gtk.gdk.Region()
-        region.union_with_rect(gtk.gdk.Rectangle(x, y, w, h))
+        region = new_region()
+        add_rectangle(region, gtk.gdk.Rectangle(x, y, w, h))
         self._damage_delayed_expired = False
         self._damage_delayed = now, window, region, self.encoding, options or {}
         debug("damage(%s, %s, %s, %s, %s) wid=%s, scheduling batching expiry for sequence %s in %.1f ms", x, y, w, h, options, self.wid, self._sequence, delay)
@@ -662,20 +680,13 @@ class WindowSource(object):
                 pixels_threshold = ww*wh/2
                 packet_cost = 4096
             pixel_count = 0
-            while not damage.empty():
-                try:
-                    x, y, w, h = get_rectangle_from_region(damage)
-                    pixel_count += w*h
-                    #favor full screen updates over many regions:
-                    if len(regions)>count_threshold or pixel_count+packet_cost*len(regions)>=pixels_threshold:
-                        send_full_screen_update()
-                        return
-                    regions.append((x, y, w, h))
-                    rect = gtk.gdk.Rectangle(x, y, w, h)
-                    damage.subtract(gtk.gdk.region_rectangle(rect))
-                except ValueError:
-                    error("send_delayed_regions: damage is empty: %s", damage)
-                    break
+            for rect in get_rectangles(damage):
+                pixel_count += rect.width*rect.height
+                #favor full screen updates over many regions:
+                if len(regions)>count_threshold or pixel_count+packet_cost*len(regions)>=pixels_threshold:
+                    send_full_screen_update()
+                    return
+                regions.append((rect.x, rect.y, rect.width, rect.height))
             debug("send_delayed_regions: to regions: %s items, %s pixels", len(regions), pixel_count)
         except Exception, e:
             error("send_delayed_regions: error processing region %s: %s", damage, e, exc_info=True)
@@ -748,14 +759,29 @@ class WindowSource(object):
 
     def process_damage_region(self, damage_time, window, x, y, w, h, coding, options):
         """
-            Called by 'damage' or 'send_delayed_regions' to process a damage region,
+            Called by 'damage' or 'send_delayed_regions' to process a damage region.
+            (here we may still generate more than one damage region processing
+             to deal with video encoders and odd window sizes)
+        """
+        self.do_process_damage_region(damage_time, window, x, y, w, h, coding, options)
+        if coding in ("vpx", "x264"):
+            if w%2==1:
+                lossless = self.find_common_lossless_encoder(coding, 1*h)
+                self.do_process_damage_region(damage_time, window, x+w-1, y, x+w-1, h, lossless, options)
+            if h%2==1:
+                lossless = self.find_common_lossless_encoder(coding, w*1)
+                self.do_process_damage_region(damage_time, window, x, y+h-1, x+w, y+h-1, lossless, options)
+
+    def do_process_damage_region(self, damage_time, window, x, y, w, h, coding, options):
+        """
+            Actual damage region processing:
             we extract the rgb data from the pixmap and place it on the damage queue.
             This runs in the UI thread.
         """
         if w==0 or h==0:
             return
         if not window.is_managed():
-            warn("the window %s is not composited!?", window)
+            debug("the window %s is not composited!?", window)
             return
         # It's important to acknowledge changes *before* we extract them,
         # to avoid a race condition.
