@@ -5,11 +5,25 @@
 
 # Platform-specific code for Posix systems with X11 display.
 import os.path
+import sys
+#preserve the spaces below to make it easier to apply patches:
+
+
 
 XPRA_LOCAL_SERVERS_SUPPORTED = True
+
+
+
+XPRA_SHADOW_SUPPORTED = True
+
+
+
 DEFAULT_SSH_CMD = "ssh"
 GOT_PASSWORD_PROMPT_SUGGESTION = "Perhaps you need to set up your ssh agent?\n"
 
+def do_init():
+    pass
+
 def add_client_options(parser):
     from xpra.platform import add_notray_option, add_delaytray_option
     add_notray_option(parser)
@@ -31,3 +45,9 @@ def get_machine_id():
             except:
                 pass
     return  v
+
+def get_app_dir():
+    return os.path.join(sys.exec_prefix, "share", "xpra")
+
+def get_icon_dir():
+    return os.path.join(get_app_dir(), "icons")
